import cachetools
import json
import hashlib
from app.db.redis import redis_client

def timed_lru_cache(seconds: int = 0, minutes: int = 0, maxsize: int = 128):
    """
    Decorator that wraps the LRU cache implementation and adds a time expiration
    """

    ttl = seconds + (minutes * 60)


    def decorator(func):
        func = cachetools.cached(
            cache=cachetools.TTLCache(maxsize=maxsize, ttl=ttl)
        )(func)

        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        return wrapper
    return decorator

def cache_in_redis(seconds: int = 0, minutes: int = 0, hours: int = 0, days: int = 0):
    ttl = seconds + (minutes * 60) + (hours * 60 * 60) + (days * 60 * 60 * 24)
    def decorator(func):
        def wrapper(*args, **kwargs):
            key = func.__name__ + hashlib.md5(str(args).encode() + str(kwargs).encode()).hexdigest()
            
            if (result := redis_client.get(key)) is not None:
                return json.loads(result)
            
            result = func(*args, **kwargs)
            redis_client.setex(key, ttl, json.dumps(result))

            return result
        return wrapper
    return decorator


import os
import shutil

from fasttemplates.constants import WORKING_DIR


def copy_files(src_dir, dst_dir, ignore_dirs=None):
    if ignore_dirs is None:
        ignore_dirs = []

    for item in os.listdir(src_dir):
        if item in ignore_dirs:
            continue

        src_item = os.path.join(src_dir, item)
        dst_item = os.path.join(dst_dir, item)

        if os.path.isdir(src_item):
            os.makedirs(dst_item, exist_ok=True)
            copy_files(src_item, dst_item, ignore_dirs)
        else:
            shutil.copy2(src_item, dst_item)


def add_mongo_uri_to_settings():
    # Add MONGO_URI to Settings class
    settings_path = os.path.join(WORKING_DIR, "app\\settings.py")
    with open(settings_path, "r") as f:
        lines = f.readlines()

    # Find the line with the class definition
    class_def_index = next(i for i, line in enumerate(
        lines) if 'class Settings(BaseSettings):' in line)

    # Insert the new line after the class definition
    lines.insert(class_def_index + 3, '    MONGO_URI:  str = None\n')

    # Write the modified lines back to the file
    with open(settings_path, "w") as f:
        f.writelines(lines)



import os

MONGO_DIR_NAME = 'mongo'
REDIS_DIR_NAME = 'redis'

MONGO_PROMPT = "MongoDB database"
REDIS_PROMPT = "Redis database"

WORKING_DIR = os.getcwd()
BASE_DIR = os.path.dirname(os.path.abspath(__file__))

"BASE LAYOUT"
BASE_LAYOUT = os.path.join(BASE_DIR, "templates\\baselayout")


"EXTRAS"
DB_TREE = os.path.join(BASE_DIR, "templates\\extras\\db")
DB_DIR = os.path.join(BASE_LAYOUT, "app\\db")
REDIS_CACHE_TREE = os.path.join(BASE_DIR, "templates\\extras\\cache\\redis")

"Repositories"
FASTAPI_FS = 'FastAPI Fullstack Template by Tiangolo'
FASTAPI_DJANGO_ORM = 'FastAPI with Django ORM by Uhttred'
FASTAPI_FULLSTACK = 'https://github.com/tiangolo/full-stack-fastapi-template.git'
FASTAPI_WITH_DJANGO_ORM = 'https://github.com/uhttred/fastapi-with-django-orm-template.git'

DOCKER_COMPOSE_YML = """    depends_on:
        - db
    environment:
        - MONGO_URI: mongodb://db:27017
  db:
    image: mongo:latest
    ports:
      - "27017:27017" # This config is to expose the port to the host machine. 
      # We could delete this line and the app would still be able to connect to the db container
    volumes:
      - mongodb_data_container:/data/db
volumes:
  mongodb_data_container:
"""



import cachetools


def timed_lru_cache(seconds: int = 0, minutes: int = 0, maxsize: int = 128):
    """
    Decorator that wraps the LRU cache implementation and adds a time expiration
    """

    ttl = seconds + (minutes * 60)

    def decorator(func):
        func = cachetools.cached(
            cache=cachetools.TTLCache(maxsize=maxsize, ttl=ttl)
        )(func)

        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        return wrapper
    return decorator



from fastapi import FastAPI
from routes import routers

app = FastAPI()
[
    app.include_router(prefix=router_dict['prefix'],
                       router=router_dict['router'])
    for router_dict in routers
]



import logging
logging.basicConfig(level=logging.INFO)

handler = logging.StreamHandler()
formatter = logging.Formatter('%(levelname)s\t%(pathname)s\t%(message)s')
handler.setFormatter(formatter)

logger = logging.getLogger(__name__)
logger.addHandler(handler)
